"""Various string functions"""
import os


def write_file(f, value):
    """Write something into a file. Overwrite if existing, create if new"""
    if isinstance(f, str):
        print("Writing file '%s'" % f)
        fh = open(f, "wb")
        fh.write(value)
    else:
        raise TypeError


def delete_file(value):
    """Delete an individual file, or a list of files"""
    if isinstance(value, list):
        for f in value:
            print("Deleting file '%s'" % f)
            os.remove(f)
    elif isinstance(value, str):
        print("Deleting file '%s'" % value)
        os.remove(value)
    else:
        raise TypeError



"""Functions to help with testing."""
import os
import re
import requests
from fabric.api import get, hide, run
from ast import literal_eval


def get_artifacts(artifacts=False, envvar='CIRCLE_ARTIFACTS'):
    """Uses Fabric to get each artifact provided by the artifacts list."""

    # Pull artifacts directory target from env vars
    directory = os.environ.get(envvar, 'tmp')

    # Set default artifacts if none are supplied
    if artifacts:
        pass
    else:
        heat_logs = ['/root/cfn-userdata.log', '/root/heat-script.log']
        cloudinit_logs = ['/var/log/cloud-init.log',
                          '/var/log/cloud-init-output.log']
        code = "python -c 'from glob import glob; \
                    print glob(\"/tmp/heat_chef/*-*-*-*-*/*.log\")'"
        chef_logs = literal_eval(run(code))
        artifacts = heat_logs + chef_logs + cloudinit_logs

    for artifact in artifacts:
        target = directory + "/%(host)s/%(path)s"
        try:
            with hide('aborts'):
                get(artifact, target)
        except:
            pass


def http_check(site, string):
    """Search the html of site with the string provided."""
    with hide('running', 'stdout'):
        wget_cmd = "wget --quiet --output-document - --no-check-certificate"
        homepage = run("{0} {1}".format(wget_cmd, site))
        if re.search(string, homepage):
            return True
        else:
            return False


def local_http_check(url, string):
    """Run a local http request and search the html for the provided string."""
    try:
        r = requests.get(url)
        if re.search(string, r.content):
            return True
        else:
            return False
    except:
        return False



import requests

RULES = [
    "TemplateLintRequiredSections",
    "TemplateLintOutputChecks",
    "TemplateLintParameterLabelCheck",
    "TemplateLintParameterDescriptionCheck",
    "TemplateLintParameterConstraintCheck",
    "TemplateLintParameterGroupLabelCheck",
    "MetadataRequiredSections",
    "MetadataReachImagesDefined",
    "MetadataReachImagesAvailable"
]


class TemplateLintRule(object):
    """Base class for template lint rules"""
    def __init__(self, template, metadata):
        self.template = template
        self.metadata = metadata
        self.custom_message = None
        self.name = "Unnamed Rule"
        self.description = "No rule description"
        self.set_name()

    def set_name(self):
        """Placeholder for derived class"""
        pass

    def passes_check(self):
        """Placeholder for derived class"""

    def __repr__(self):
        return "TemplateLintRule(name=%s, description=%s)" % (
            self.name, self.description)

    def message(self, message):
        """Set a custom message"""
        self.custom_message = "%s: %s. %s" % (self.name, self.description,
                                              message)

    def check(self):
        """Prints a rule message if the check fails"""
        if not self.passes_check():
            if self.custom_message:
                print(self.custom_message)
            else:
                print("%s: %s" % (self.name, self.description))


class TemplateLintRequiredSections(TemplateLintRule):
    """Require sections of a template"""
    def set_name(self):
        self.name = "LINT-001"
        self.description = "`heat_template_version`, `description`, "\
                           "`parameter_groups`, `parameters`, `resources`, "\
                           "and `outputs` sections are required."

    def passes_check(self):
        required_sections = ['heat_template_version', 'description',
                             'parameter_groups', 'parameters', 'resources',
                             'outputs']
        for section in required_sections:
            if section not in self.template:
                return False
        return True


class TemplateLintOutputChecks(TemplateLintRule):
    """Verify how all outputs are setup"""
    def set_name(self):
        self.name = "LINT-002"
        self.description = "All outputs should have a `description` and "\
                           "`value` defined ."

    def passes_check(self):
        required_keys = ['description', 'value']
        outputs = self.template['outputs']
        for output, values in outputs.items():
            for key in required_keys:
                if key not in values:
                    return False
        return True


class TemplateLintParameterLabelCheck(TemplateLintRule):
    """Verify that all parameters have a label"""
    def set_name(self):
        self.name = "LINT-003"
        self.description = "All parameters should have a defined `label`."

    def passes_check(self):
        parameters = self.template['parameters']
        for parameter, values in parameters.items():
            if 'label' not in values:
                return False
        return True


class TemplateLintParameterDescriptionCheck(TemplateLintRule):
    """Verify that all parameters have a description"""
    def set_name(self):
        self.name = "LINT-004"
        self.description = "All parameters should have a defined "\
                           "`description`."

    def passes_check(self):
        parameters = self.template['parameters']
        for parameter, values in parameters.items():
            if 'description' not in values:
                return False
        return True


class TemplateLintParameterConstraintCheck(TemplateLintRule):
    """Verify that all parameters have a constraint"""
    def set_name(self):
        self.name = "LINT-005"
        self.description = "All parameters should have defined "\
                           "`constraints`."

    def passes_check(self):
        parameters = self.template['parameters']
        for parameter, values in parameters.items():
            if 'constraints' not in values:
                return False
        return True


class TemplateLintParameterGroupLabelCheck(TemplateLintRule):
    """Verify that all parameter groups have a label"""
    def set_name(self):
        self.name = "LINT-006"
        self.description = "All parameters groups should have a `label`."

    def passes_check(self):
        parameter_groups = self.template['parameter_groups']
        for group in parameter_groups:
            if 'label' not in group:
                return False
        return True


class MetadataRequiredSections(TemplateLintRule):
    """Verify all required metadata sections are present"""
    def set_name(self):
        self.name = "LINT-007"
        self.description = "`schema-version`, `application-family`, "\
                           "`application-name`, `application-version`, "\
                           "`flavor`, `flavor-weight`, `reach-info`, "\
                           "`abstract`, and `instructions` sections are "\
                           "required in the metadata."

    def passes_check(self):
        required_sections = ['schema-version', 'application-family',
                             'application-name', 'application-version',
                             'flavor', 'flavor-weight', 'reach-info',
                             'abstract', 'instructions']
        for section in required_sections:
            if section not in self.metadata:
                return False
        return True


class MetadataReachImagesDefined(TemplateLintRule):
    """Verify that all reach images exist"""
    def set_name(self):
        self.name = "LINT-008"
        self.description = "`tattoo` and `icon-20x20` images should be "\
                           "defined."

    def passes_check(self):
        images = ['tattoo', 'icon-20x20']
        for image in images:
            if image not in self.metadata['reach-info']:
                return False
        return True


class MetadataReachImagesAvailable(TemplateLintRule):
    """Verify that all reach images defined are available"""
    def set_name(self):
        self.name = "LINT-009"
        self.description = "`tattoo` and `icon-20x20` images should be "\
                           "accessible."

    def passes_check(self):
        images = ['tattoo', 'icon-20x20']
        for image in images:
            if image in self.metadata['reach-info']:
                img = requests.get(self.metadata['reach-info'][image])
                if not img.ok:
                    return False
        return True



"""Various string functions"""


def list_to_string(values, join_char=" "):
    """Joins a list of strings and returns a string joined by a given string"""
    if isinstance(values, list):
        return join_char.join(str(value) for value in values)
    else:
        return values


def string_to_list(value):
    """Converts a space delimited string into a list"""
    if isinstance(value, str):
        return value.split()
    else:
        return value


def valid_project_name(name):
    """Validate that the project name would be a valid GitHub project name.
    GitHub appears to be super flexible on naming. It accepts special
    characters and spaces. From some testing, it appears the only limit is
    that the string needs to be under 100 characters."""
    if isinstance(name, str) and len(name) < 100:
        return True
    return False




import os
import keystoneclient.v2_0.client as ksclient


class OSAuth(object):

    def __init__(self):
        self.creds = self.get_keystone_creds()
        self.keystone_client = ksclient.Client(**self.creds)
        if not self.creds.get('region_name'):
            sc = self.keystone_client.service_catalog.catalog
            self.creds['region_name'] = \
                sc['user'].get('RAX-AUTH:defaultRegion')

    def get_keystone_creds(self):
        creds = {}
        if os.environ.get('OS_AUTH_TOKEN'):
            creds['auth_url'] = os.environ['OS_AUTH_URL']
            creds['token'] = os.environ['OS_AUTH_TOKEN']
        else:
            creds['username'] = os.environ['OS_USERNAME']
            creds['password'] = os.environ['OS_PASSWORD']
            creds['auth_url'] = os.environ['OS_AUTH_URL']
            creds['tenant_id'] = os.environ['OS_TENANT_ID']
            creds['region_name'] = os.environ.get('OS_REGION_NAME')
            if creds.get('region_name'):
                creds['region_name'] = creds['region_name'].upper()
        return creds

    def get_token(self):
        return self.keystone_client.auth_token

    def get_heat_url(self):
        if os.environ.get('HEAT_URL'):
            return os.environ.get('HEAT_URL')
        else:
            region = self.creds.get('region_name')
            heat_url = self.keystone_client.service_catalog.url_for(
                service_type='orchestration',
                endpoint_type='publicURL', region_name=region)
            return heat_url



import fabric
import imp
import os

from fabric.api import env


def run_fabric_tasks(test_name, test):
    """Setup fabric environment and run fabric script"""
    env_setup = test['fabric']
    if env_setup['env']:
        fab_file = env_setup['env']['fabfile']
        print("  Preparing environtment to run fabric tests:")
        for k, v in env_setup['env'].iteritems():
            print("    Setting env['%s'] to %s" % (k, v))
            env[k] = v
        mod_name = os.path.splitext(os.path.basename(fab_file))[0]
        mod = imp.load_source(mod_name, fab_file)
        for task in env_setup['env']['tasks']:
            print("  Run fabric test '%s', task '%s' on: %s" % (test_name,
                                                                task,
                                                                env.hosts))
            fabric.tasks.execute(getattr(mod, task))



import os
import subprocess

from hot.utils import files
from hot.utils import hosts
from hot.utils import string


def run_script(test_name, test):
    """Setup environment and run a script"""
    script_setup = test['script']
    if "hosts" in script_setup:
        if os.name == 'nt':
            hosts_path = os.path.join(os.environ['SYSTEMROOT'],
                                      'system32/drivers/etc/hosts')
        elif os.name == 'posix':
            hosts_path = '/etc/hosts'
        h = hosts.Hosts(hosts_path)
        for entry in script_setup['hosts']:
            if "ip" in entry:
                ip = entry["ip"]
                if "hostnames" in entry:
                    for hostname in entry['hostnames']:
                        print("  Setting host entry '%s\t%s'" % (ip, hostname))
                        if isinstance(hostname, str):
                            h.set_one(hostname, ip)
                        elif isinstance(hostname, list):
                            h.set_all(hostname, ip)
            else:
                raise TypeError
        h.write(hosts_path)

    # Write Files
    if "files" in script_setup:
        for file, value in files:
            files.write_file(file, value)

    # Set Environment Variables
    if "environment" in script_setup:
        for k, v in script_setup["environment"]:
            os.environ[k] = str(v)

    # Run commands
    if "commands" in script_setup:
        # TODO: Run scripts
        for cmd in script_setup["commands"]:
            if "command" in cmd:
                command = cmd["command"]
                if "command_args" in cmd:
                    args = cmd['command_args']
                    command = " ".join((command, string.list_to_string(args)))
                cmd_list = string.string_to_list(command)
                subprocess.check_call(cmd_list)



"""Misc utilities that can be used across the project"""

import files
import hosts
import repo
import test
import timeout
import token
import string
import yaml

__all__ = ["files",
           "hosts",
           "repo",
           "test",
           "timeout",
           "token",
           "string",
           "yaml"]
